# This script is for printing debug information in CI, to help with tracking
# down configuration errosr. It's similar to what PyO3 use to detect the Python
# version.

import sys
import sysconfig
import platform
import json

PYPY = platform.python_implementation() == "PyPy"

try:
    base_prefix = sys.base_prefix
except AttributeError:
    base_prefix = sys.exec_prefix

info = {
    "version": {
        "major": sys.version_info[0],
        "minor": sys.version_info[1],
        "implementation": platform.python_implementation()
    },
    "libdir":
    sysconfig.get_config_var('LIBDIR'),
    "ld_version":
    sysconfig.get_config_var('LDVERSION')
    or sysconfig.get_config_var('py_version_short'),
    "base_prefix":
    base_prefix,
    "shared":
    PYPY or bool(sysconfig.get_config_var('Py_ENABLE_SHARED')),
    "executable":
    sys.executable,
    "machine":
    platform.machine(),
    "maxsize_bits":
    sys.maxsize.bit_length(),
    "architecture":
    platform.architecture(),
    "platform":
    platform.platform(),
}

print(json.dumps(info, indent="  "))



#! /usr/bin/env python3

from pathlib import Path
import shutil
import subprocess

ROOT = Path(__file__).parent.parent.parent

# Workaround for https://github.com/PyO3/maturin/pull/27
shutil.copy2(ROOT / "Cargo.toml", ROOT / "Cargo.toml.orig")

subprocess.run(["maturin", "sdist"])

sdists = [x for x in (ROOT / "target" / "wheels").iterdir()]
if len(sdists) != 1:
    raise RuntimeError("expected one sdist, found " + repr(sdists))

print("::set-output name=sdist_path::" + str(sdists[0]))
print("::set-output name=sdist_name::" + sdists[0].name)



#! /usr/bin/env python3

# Build the shared library, and then copy it into this directory, so that
# test_blake3.py can see it. We use the filename that Python requires on the
# curent platform: blake3.so on Linux and macOS, and blake3.pyd on Windows.

import os
from pathlib import Path
import platform
import shutil
import subprocess

HERE = Path(__file__).parent
ROOT = HERE / ".."

# We don't set --target here, but CI sets CARGO_BUILD_TARGET, which has the
# same effect.
subprocess.run(["cargo", "build", "--release"], check=True, cwd=str(ROOT))

if platform.system() == "Windows":
    SRC_NAME = "blake3.dll"
    DEST_NAME = "blake3.pyd"
elif platform.system() == "Darwin":
    SRC_NAME = "libblake3.dylib"
    DEST_NAME = "blake3.so"
else:
    # Assume everything else behaves like Linux.
    SRC_NAME = "libblake3.so"
    DEST_NAME = "blake3.so"

source_path = ROOT / "target"
cargo_build_target = os.environ.get("CARGO_BUILD_TARGET")
if cargo_build_target:
    source_path = source_path / cargo_build_target
source_path = source_path / "release" / SRC_NAME
destination_path = HERE / DEST_NAME
print("copying", source_path, "to", destination_path)
shutil.copy2(str(source_path), str(destination_path))



#! /usr/bin/env python3

import os
import platform
import sys
from pathlib import Path
import subprocess

ROOT = Path(__file__).parent.parent.parent

# For macOS and Windows, we run Maturin against the Python interpreter that's
# been installed and configured for this CI run, i.e. the one that's running
# this script. (There are generally several versions installed by default, but
# that's not guaranteed.) For Linux, in order to get "manylinux" compatibility
# right, we need to run Maturin in a special Docker container. We hardcode
# paths to specific interpreter versions, based on where things are installed
# in this container. Our GitHub config has no effect on the the container, so
# we could build all the wheels in one job, but we stick to one-wheel-per-job
# for consistency.
if platform.system() == "Linux":
    version_path_components = {
        (3, 6): "cp36-cp36m",
        (3, 7): "cp37-cp37m",
        (3, 8): "cp38-cp38",
        (3, 9): "cp39-cp39",
        # This list needs to be kept in sync with push.yml and tag.yml.
    }
    version_component = version_path_components[sys.version_info[:2]]
    interpreter_path = "/opt/python/" + version_component + "/bin/python"
    # See https://github.com/PyO3/maturin#manylinux-and-auditwheel
    command = [
        "docker",
        "run",
        "--rm",
        "--volume=" + os.getcwd() + ":/io",
        "--env=BLAKE3_CI=1", # don't allow fallbacks for missing AVX-512 support
        "konstin2/maturin",
        "build",
        "--release",
        "--no-sdist",
        "--interpreter",
        interpreter_path,
    ]
    subprocess.run(command, check=True)
else:
    command = [
        "maturin",
        "build",
        "--release",
        "--no-sdist",
        "--interpreter",
        sys.executable,
    ]
    subprocess.run(command, check=True)

wheels = [x for x in (ROOT / "target" / "wheels").iterdir()]
if len(wheels) != 1:
    raise RuntimeError("expected one wheel, found " + repr(wheels))

print("::set-output name=wheel_path::" + str(wheels[0]))
print("::set-output name=wheel_name::" + wheels[0].name)



#! /usr/bin/env python3

import github
import os
import shutil
import subprocess
import tempfile
import urllib

g = github.Github()  # read-only, no token needed
tag_name = os.environ["GITHUB_TAG"]
tag_prefix = "refs/tags/"
if tag_name.startswith(tag_prefix):
    tag_name = tag_name[len(tag_prefix):]
rerelease_suffix = "_rerelease"
if tag_name.endswith(rerelease_suffix):
    tag_name = tag_name[:len(tag_name) - len(rerelease_suffix)]
    print("This is a rerelease of {}.".format(tag_name))

repo = g.get_repo("oconnor663/blake3-py")

releases = list(repo.get_releases())
for release in releases:
    if release.tag_name == tag_name:
        break
else:
    raise RuntimeError("no release for tag " + repr(tag_name))

asset_names = [asset.name for asset in release.get_assets()]
asset_files = []

tempdir = tempfile.mkdtemp()
for asset_name in asset_names:
    urlbase = "https://github.com/oconnor663/blake3-py/releases/download/{}/{}"
    url = urlbase.format(tag_name, asset_name)
    filepath = os.path.join(tempdir, asset_name)
    asset_files.append(filepath)
    with urllib.request.urlopen(url) as request, open(filepath, "wb") as f:
        print("Downloading " + asset_name)
        shutil.copyfileobj(request, f)

print("Uploading to PyPI with twine...")
# Credentials are in the environment, TWINE_USERNAME and TWINE_PASSWORD.
twine_cmd = ["twine", "upload", "--skip-existing"] + asset_files
subprocess.run(twine_cmd, check=True)

print("Success!")
